/* GPIO Example

   This example code is in the Public Domain (or CC0 licensed, at your option.)

   Unless required by applicable law or agreed to in writing, this
   software is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
   CONDITIONS OF ANY KIND, either express or implied.
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/queue.h"
#include "driver/gpio.h"
#include "driver/uart.h"
#include "motors.h"
#include "encoders.h"
#include "hwTimer.h"
#include "control.h"
#include "esp_log.h"

#define OMNI_CTRL_TIMER 0.05 // s
#define MAX_OUTPUT 100.0
#define NMOTORS 3
#define OMNI_NRO_TIMER 0

static const char *tag = "UART EVENT";
#define UART_NUM UART_NUM_2
#define FLOAT_BUF_SIZE 3
#define BUF_SIZE FLOAT_BUF_SIZE * sizeof(float)
#define TASK_MEMORY 1024 * 2

static QueueHandle_t uart_queue;
static uint8_t data[BUF_SIZE];
static float fdata[FLOAT_BUF_SIZE];

static void uart_task(void *pvParameters)
{
    uart_event_t event;
    int len;
    while (1)
    {
        if (xQueueReceive(uart_queue, &event, portMAX_DELAY ) )
        {
            bzero(data,BUF_SIZE);
            switch (event.type)
            {
            case UART_DATA:
                len = uart_read_bytes(UART_NUM, data, BUF_SIZE, portMAX_DELAY);
                if (len == BUF_SIZE){
                    memcpy(&fdata, data , sizeof(fdata));
                    omni_drive(fdata[0], fdata[1], fdata[2], 0);
                    uart_flush(UART_NUM);
                }else{
                    printf("Todavia no se completo el mensaje o hay un error. \n");
                }
                break;
            default:
                ESP_LOGI(tag, "uart event type: %d", event.type);
                break;
            }
        }   
    }   
}

static void init_uart(void)
{
    uart_config_t uart_config = {
        .baud_rate = 9600,
        .data_bits = UART_DATA_8_BITS,
        .parity = UART_PARITY_DISABLE,
        .stop_bits = UART_STOP_BITS_2,
        .flow_ctrl = UART_HW_FLOWCTRL_DISABLE,
        .source_clk = UART_SCLK_APB,
    };
    uart_param_config(UART_NUM, &uart_config);

    uart_set_pin(UART_NUM, 0, 4, UART_PIN_NO_CHANGE, UART_PIN_NO_CHANGE);

    uart_driver_install(UART_NUM, 1024, 1024, 5, &uart_queue, 0);
    xTaskCreate(uart_task, "uart_task", TASK_MEMORY, NULL, 5, NULL);
}

void app_main(void)
{
    init_uart();
    printf("Configuring Initial Parameters of mcpwm...\n");
    omni_init();
    omni_set_enable(true);
    while (1) {   
        vTaskDelay( portMAX_DELAY);
    } 
}